@@ -1,11 +1,18 @@
 /*
-#  $Id: Driver_xst.h 2488 2006-02-07 22:24:43Z timbo $
+#  $Id: Driver_xst.h 15062 2012-01-02 15:16:55Z timbo $
 #  Copyright (c) 2002  Tim Bunce  Ireland
 #
 #  You may distribute under the terms of either the GNU General Public
 #  License or the Artistic License, as specified in the Perl README file.
 */
 
+
+/* This is really just a workaround for SUPER:: not working right for XS code.
+ * It would be better if we setup perl's context so SUPER:: did the right thing
+ * (borrowing the relevant magic from pp_entersub in perl pp_hot.c).
+ * Then we could just use call_method("SUPER::foo") instead.
+ * XXX remember to call SPAGAIN in the calling code after calling this!
+ */
 static SV *
 dbixst_bounce_method(char *methname, int params)
 {
@@ -26,13 +33,13 @@ dbixst_bounce_method(char *methname, int params)
     EXTEND(SP, params);
     PUSHMARK(SP);
     for (i=0; i < params; ++i) {
-	sv = (i >= items) ? &sv_undef : ST(i);
+	sv = (i >= items) ? &PL_sv_undef : ST(i);
         PUSHs(sv);
     }
     PUTBACK;
-    i = perl_call_method(methname, G_SCALAR);
+    i = call_method(methname, G_SCALAR);
     SPAGAIN;
-    sv = (i) ? POPs : &sv_undef;
+    sv = (i) ? POPs : &PL_sv_undef;
     PUTBACK;
     if (debug >= 3)
 	PerlIO_printf(DBIc_LOGPIO(imp_xxh),
@@ -54,10 +61,15 @@ dbdxst_bind_params(SV *sth, imp_sth_t *imp_sth, I32 items, I32 ax)
 	&& DBIc_NUM_PARAMS(imp_sth) != DBIc_NUM_PARAMS_AT_EXECUTE
     ) {
 	char errmsg[99];
+        /* clear any previous ParamValues before error is generated */
+        SV **svp = hv_fetch((HV*)DBIc_MY_H(imp_sth),"ParamValues",11,FALSE);
+        if (svp && SvROK(*svp) && SvTYPE(SvRV(*svp)) == SVt_PVHV) {
+            HV *hv = (HV*)SvRV(*svp);
+            hv_clear(hv);
+        }
 	sprintf(errmsg,"called with %d bind variables when %d are needed",
 		(int)items-1, DBIc_NUM_PARAMS(imp_sth));
-	sv_setpv(DBIc_ERRSTR(imp_sth), errmsg);
-	sv_setiv(DBIc_ERR(imp_sth), (IV)-1);
+        DBIh_SET_ERR_CHAR(sth, imp_sth, "-1", -1, errmsg, Nullch, Nullch);
 	return 0;
     }
     idx = sv_2mortal(newSViv(0));
@@ -83,9 +95,8 @@ dbdxst_fetchall_arrayref(SV *sth, SV *slice, SV *batch_row_count)
     if (SvOK(slice)) {  /* should never get here */
 	char errmsg[99];
 	sprintf(errmsg,"slice param not supported by XS version of fetchall_arrayref");
-	sv_setpv(DBIc_ERRSTR(imp_sth), errmsg);
-	sv_setiv(DBIc_ERR(imp_sth), (IV)-1);
-	return &sv_undef;
+        DBIh_SET_ERR_CHAR(sth, imp_sth, "-1", -1, errmsg, Nullch, Nullch);
+	return &PL_sv_undef;
     }
     else {
 	IV maxrows = SvOK(batch_row_count) ? SvIV(batch_row_count) : -1;
@@ -94,7 +105,7 @@ dbdxst_fetchall_arrayref(SV *sth, SV *slice, SV *batch_row_count)
 	if ( !DBIc_ACTIVE(imp_sth) && maxrows>0 ) {
 	    /* to simplify application logic we return undef without an error	*/
 	    /* if we've fetched all the rows and called with a batch_row_count	*/
-	    return &sv_undef;
+	    return &PL_sv_undef;
 	}
 	av_extend(rows_av, (maxrows>0) ? maxrows : 31);
 	while ( (maxrows < 0 || maxrows-- > 0)
